@@ -3,6 +3,7 @@ ConfigConvert is a set of ConversionKit converters designed specifically to
 help you create Python objects from options set in a text file.
 """
 
+import getopt
 import logging
 import os
 
@@ -10,6 +11,8 @@ log = logging.getLogger(__name__)
 
 from configconvert.internal import eval_import, simple_import, import_module
 from bn import uniform_path as up
+from nestedrecord import decodeNestedRecord, encode_error
+from conversionkit import Conversion
 
 #
 # ConversionKit handlers
@@ -33,6 +36,9 @@ def stringToObject():
             conversion.result = result
     return stringToObject_converter
 
+# XXX Quick hack.
+unicodeToObject=stringToObject
+
 #
 # File Handling Converters
 #    
@@ -41,6 +47,7 @@ def existingDirectory(
     try_to_create=False, 
     raise_on_create_error=False,
     uniform_path=False,
+    include_value_in_error=True,
 ):
     """\
     Ensure the directory specified as the input exists and is not a file, 
@@ -65,18 +72,27 @@ def existingDirectory(
                 except:
                     if raise_on_create_error:
                         raise
-                    conversion.error = 'Could not create the directory %r' % (
-                        directory
-                    )
+                    if include_value_in_error:
+                        conversion.error = 'Could not create the directory %r' % (
+                            directory
+                        )
+                    else:
+                        conversion.error = 'Could not create the directory'
                 else:
                     if uniform_path:
                         conversion.result = up(directory)
                     else:
                         conversion.result = directory
             else:
-                conversion.error = 'The directory doesn\'t exist'
+                if include_value_in_error:
+                    conversion.error = 'The directory %r doesn\'t exist'%directory
+                else:
+                    conversion.error = 'The directory doesn\'t exist'
         elif not os.path.isdir(directory):
-            conversion.error = 'The path is not a directory'
+            if include_value_in_error:
+                conversion.error = 'The path %r is not a directory'%directory
+            else:
+                conversion.error = 'The path is not a directory'
         else:
             if uniform_path:
                 conversion.result = up(directory)
@@ -89,6 +105,7 @@ def existingFile(
     raise_on_create_error=False, 
     file_content='',
     uniform_path=False,
+    include_value_in_error=True,
 ):
     """\
     Ensure the file specified as the input exists and is not a directory, 
@@ -125,18 +142,27 @@ def existingFile(
                 except:
                     if raise_on_create_error:
                         raise
-                    conversion.error = 'Could not create the file %r' % (
-                        filename
-                    )
+                    if include_value_in_error:
+                        conversion.error = 'Could not create the file %r' % (
+                            filename
+                        )
+                    else:
+                        conversion.error = 'Could not create the file'
                 else:
                     if uniform_path:
                         conversion.result = up(filename)
                     else:
                         conversion.result = filename
             else:
-                conversion.error = 'The file doesn\'t exist'
+                if include_value_in_error:
+                    conversion.error = 'The file %r doesn\'t exist'%filename
+                else:
+                    conversion.error = 'The file doesn\'t exist'
         elif os.path.isdir(filename):
-            conversion.error = 'The path is not a file'
+            if include_value_in_error:
+                conversion.error = 'The path %r is not a file'%filename
+            else:
+                conversion.error = 'The path is not a file'
         else:
             if uniform_path:
                 conversion.result = up(filename)
@@ -164,7 +190,7 @@ def parse_config(filename, encoding='utf8'):
 
     The config file is made of options and values which are each defined on a
     single line terminated by a ``\n`` character and separtated by only the 
-    three characters `` = ``. For example
+    three characters *space equals space*. For example:
 
     ::
 
@@ -173,30 +199,34 @@ def parse_config(filename, encoding='utf8'):
     The parsed config file results in a dictionary with the options as ASCII
     strings for the keys and the values as unicode strings for the values. The
     options must start with the letters a-z, A-Z or _ and should contain only
-    letters, numbers or the ``_`` character. Thus the option values
-    have the same naming rules as Python variables.
+    letters, numbers or the ``_`` character. Thus the option values have the
+    same naming rules as Python variables.
 
-    If you don't leave a space each side of the ``=`` character it is considered
-    a syntax error. Any extra space characters after the space
-    after the equals sign are treated as part of the value. For example this::
+    If you don't leave a space each side of the ``=`` character it is
+    considered a syntax error. Any extra space characters after the space
+    after the equals sign are treated as part of the value. For example this:
+
+    ::
 
         option =  value
    
     would result in the option ``'option'`` taking the value ``u' value'``. 
     Any extra spaces after the option name are ignored though.
 
-    The file must use UNIX style line endings (ie each line ends in ``\n``) and
-    must be encoded as UTF-8. Values can therefore take any Unicode character as
-    long as the file is encoded correctly.
+    The file must use UNIX style line endings (ie each line ends in ``\n``)
+    and must be encoded as UTF-8. Values can therefore take any Unicode
+    character as long as the file is encoded correctly.
 
-    You can also specify multiline values. You do so by specifying the first 
-    line of the multiline value on the same line as the option starting 
-    immediately after the space after the equals sign (once again any extra spaces
-    will be treated as part of the value). All subsequent lines have to be 
-    indented 4 spaces. Any characters after those 4 spaces are treated as part
-    of the line. In fact the first line doesn't have to contain any text if you
-    are using a multiline value, in which case the value will start with a ``\n``
-    character. Here are two examples::
+    You can also specify multiline values. You do so by specifying the first
+    line of the multiline value on the same line as the option starting
+    immediately after the space after the equals sign (once again any extra
+    spaces will be treated as part of the value). All subsequent lines have to
+    be indented 4 spaces. Any characters after those 4 spaces are treated as
+    part of the line. In fact the first line doesn't have to contain any text
+    if you are using a multiline value, in which case the value will start
+    with a ``\n`` character. Here are two examples:
+
+    ::
 
         option1 = This
             is
@@ -220,7 +250,7 @@ def parse_config(filename, encoding='utf8'):
         fp.close()
         fp = None
     finally:
-        if fp:
+        if 'fp' in locals() and fp:
             fp.close()
         if data is None:
             raise Exception(
@@ -262,7 +292,7 @@ def parse_config(filename, encoding='utf8'):
                  error = "Expected the characters ' = ' on line %s"
                  if '=' in line:
                      error += ", not just an '=' on its own"
-                 raise SyntaxError(error%i+1)
+                 raise SyntaxError(error%(i+1))
              elif len(parts) > 2:
                  value = ' = '.join(parts[1:])
              else:
@@ -290,23 +320,54 @@ def parse_config(filename, encoding='utf8'):
     #    conf['app.filename'] = filename
     return conf
 
-class ConfigFileError(Exception):
+class ConfigFileError(getopt.GetoptError):
     pass
 
-def handle_option_error(conversion):
+def handle_option_error(conversion, base=None):
+    if base:
+        base = base+'.'
+    else:
+        base = ''
     errors = []
-    for name, child in conversion.children.items():
-        if not child.successful:
-            errors.append(child.error)
+    for name, message in encode_error(conversion).items():
+        errors.append(base+name+': '+message%dict(name=base[:-1]))
     error = conversion.error + '\n    ' + ('\n    '.join(errors))
+    log.error(error)
     raise ConfigFileError(error)
 
-def handle_section_error(flow, option, example):
-    raise  ConfigFileError(
+def handle_section_error(bag, option, example):
+    if bag and not bag.app.option:
+        raise ConfigFileError(
+            'No config options were found, have you specified a config '
+            'file containing %r options, e.g. %s'%(
+                option, 
+                example,
+            )
+        )
+    raise ConfigFileError(
         'Expected the config file to contain %r options, e.g. %s'%(
             option, 
             example,
         )
     )
 
+def split_options(options):
+    return Conversion(options).perform(
+        decodeNestedRecord(depth=1)
+    ).result
+
+def parse_and_split_options_change_dir(service, filename):
+    """\
+    This function is where you can set up your settings. Ordinarily, 
+    settings which you want end users to configure should go in the 
+    config file, settings which the developer needs control over can
+    go here.
+    """
+    if not os.path.exists(filename):
+        raise getopt.GetoptError('No such file %r'%filename)
+    os.chdir(os.path.join(*os.path.split(os.path.abspath(filename))[:-1]))
+    options = parse_config(filename)
+    # Split up the setting into parts
+    option_group = split_options(options)
+    return option_group
 
